################################################################################
# Created       : Martin Foster
# Modified      : 06/04/2002
################################################################################
#
# Comm - Module part of Webchat designed to prompt and handle posts.
# Copyright (C) 2000-2002  Martin Foster
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#
# Author of this script can be contacted at the following:
#       E-Mail  : martin@ethereal-realms.org
#       Address : 204, 817 5TH ST NE
#                 Calgary, Alberta
#                 T2E 3W9
#
################################################################################

package Ethereal::Comm;						# Pacakace name

use Ethereal::Date;						# Date hander
use Ethereal::Filter;						# Filter system
use Ethereal::Option;						# Options handling


################################################################################
# Sub-Routines
################################################################################

#####################
# Constructor
#
# Allows to class to be exported and thus spawn objects as a result

sub new
{
	#####################
	# Data Members
	my $class    = shift;					# Self reference
	my $database = shift;					# Database handle
	my $cgi      = shift;					# CGI Handle

	my $self     = {
		HANDLE  => $database,
		CGI     => $cgi
 	 };


	#####################
	# Program Area

	# Bless $self into designated class
	bless $self, $class;

	# Return self
	return $self;
}



#####################
# Comm
#
# Subroutine primarily charged with displaying input area.   This will also call
# CommHandler which will handle the requests to determin the need for a new
# database entry

sub Comm
{
	#####################
	# Data members
	my $self     = shift;					# Self referencing link
	my $param    = shift;					# Parameter handle
	my $choice   = shift;					# Choice of the matter

	my $database = $self->{'HANDLE'};			# Database handle
	my $cgi      = $self->{'CGI'};				# CGI handle

	my $js;							# Javascript header
	my $jscode;						# Javascript code
 
	my $panel;						# Communications panel

	my $basic;						# Basic parameter list
	my $inline;						# Inline parameter list
	my $url      = $cgi->url();				# URL of script

	my $puppet;						# Puppet name
	my $color;						# Colour default
	my $locate;						# Location
	my $face;						# Font face
	my $size;						# Size default

	my $locale;						# Default settings

	my @color;						# Colours
	my @face;						# Font face
	my @locale;						# Locale settings
	my @size;						# Size fonts
	my @char;						# Character

	my %realm;						# Realm hash


	#####################
	# Program area

	# Link with Hash
	$database->GetHashRealm(\%realm);

	# Locale setting
	($locale) = $database->DataGetRealmLocale($param->{'ROOM'});


	# Populate arrays
	@color = FontSelectColor($self, $param);
	@size  = FontSelectSize($self, $param);
	@face  = FontSelectFace($self, $param);

	# Sorting
	@color = sort(@color);


	# Populate puppet list
	$database->GetListPuppetRealm(\@char, $param->{'USER'}, $param->{'ROOM'});
	

	# Store character name
	$puppet = $param->{'CHAR'};

	# Parameter cleanup
	$param->Cleanup(
	  'COLOR',
	  'COMM',
	  'FACE',
	  'POST',
	  'SIZE',
	  $realm{'TxtSubmitButton'}
	 );


	# Inline parameter handling
	# Basic inline
	$basic  = $param->EmbedInline($cgi, 'USER', $param->{'USER'}, 'CRYPT', $param->{'CRYPT'});

	# Normal use inline
	$inline = $param->EmbedInline($cgi, %{$param});


	# Retreive size and color
	($size, $color, $face) = $database->DataGetFont($puppet, $param->{'USER'}, $color[0], $face[0]);

	
	# Independent HTML
	if ($choice != 0)
	{
		# Print file and HTML header
		print $cgi->header(-target=>'Comm');
		print $database->FetchRealmHeader($param->{'ROOM'});
	}

	# Javascript handling
	# Determine need for javascript
	($js) = $database->DataGetJavascript($param->{'USER'});

	# Print if javascript is desired
	if ($js eq 'yes')
	{ 
		# Insert javascript header
		$database->DocumentGetJavascript($Database); 
	}

	# Generate javascript code
	$jscode = ($js eq 'yes') ? "ONCLICK='window.open(this.href,this.target,settings); return false'" : '';


	# Retreive Communications Panel
	$panel = $database->FetchRealmPanel($param->{'ROOM'});


	# Replace links
	# Consisent
	$panel =~ s/LPORTAL/HREF=\"$realm{'LnkPortal'}$basic\" TARGET=\"_BLANK\"/gs;

	# Standard links
 	$panel =~ s/LBOARDS/HREF=\"$realm{LnkBoard}$inline\"   TARGET=\"REALMS\" $jscode/gs;
 	$panel =~ s/LCOLORS/HREF=\"$realm{LnkColors}$inline\"  TARGET=\"REALMS\" $jscode/gs;
 	$panel =~ s/LIGNORE/HREF=\"$realm{LnkIgnore}$inline\"  TARGET=\"REALMS\" $jscode/gs;
 	$panel =~ s/LIMAILS/HREF=\"$realm{LnkMail}$basic\"     TARGET=\"REALMS\" $jscode/gs;
 	$panel =~ s/LWHOSON/HREF=\"$realm{LnkWho}$inline\"     TARGET=\"REALMS\" $jscode/gs;

	# Disjoined
 	$panel =~ s/LCONFIG/HREF=\"$realm{LnkPuppet}$inline\"  TARGET=\"CONFIG\" $jscode/gs;
 	$panel =~ s/LDIEROL/HREF=\"$realm{LnkRoll}$inline\"    TARGET=\"DIEROL\" $jscode/gs;
 	$panel =~ s/LPRIVTE/HREF=\"$realm{LnkPrivate}$inline\" TARGET=\"PRIVTE\" $jscode/gs;


	# Widgets
	# Assigning values
	my $wpostsend = $cgi->submit($realm{'TxtSubmitButton'});
	my $wposttext = $cgi->textarea(-name=>'POST',-rows=>3,-columns=>50,-wrap=>'SOFT');
	my $wcharslct = $cgi->popup_menu('CHAR', \@char, $puppet);
	my $wfontcolr = $cgi->popup_menu('COLOR', \@color, $color);
	my $wfontface = $cgi->popup_menu('FACE', \@face, $face);
	my $wfontsize = $cgi->popup_menu('SIZE', \@size, $size);

	# Scroll back functionality
	my $wscrlback = ($choice < 1) ? $cgi->checkbox(-name=>'SCROLLBACK', -label=>$realm{'TxtScrollback'}) : '&nbsp;';
	my $wscrlctrl = ($choice < 1) ? $cgi->checkbox(-name=>'GETPOSTS',   -label=>$realm{'TxtRemote'})     : '&nbsp;';


	# Locale settings
	if ($locale ne 'none')
	{
		# Bring into play
		my $wcharlocl;

		# Current location
		($locate) = $database->DataGetPuppetLocale($puppet);


		# Strict list
		if ($locale eq 'strict')
		{
			# Location defined
			$locate = (defined($locate)) ? $locate : $realm{'TxtNone'};


			# Pull list of locales
			$database->GetListRealmLocale(\@locale, $param->{'ROOM'});

			# Append default
			unshift(@locale, $realm{'TxtNone'});

			# Generate widget
			$wcharlocl = $cgi->popup_menu('LOCL', \@locale, $locate);
		}

		# Simple textfield
		else
		{
			# Location defined
			$locate = (defined($locate)) ? $locate : '';

			# Generate widget
			$wcharlocl = $cgi->textfield('LOCL', $locate, 25, 25);
		}

		# Replace macro
		$panel =~ s/WCHARLOCL/$wcharlocl/gs;
	}


	# Widget replacements
	# Posting widgets
	$panel =~ s/WPOSTSEND/$wpostsend/gs;
	$panel =~ s/WPOSTTEXT/$wposttext/gs;

	# Puppet selection
	$panel =~ s/WCHARSLCT/$wcharslct/gs;

	# Font selection
	$panel =~ s/WFONTCOLR/$wfontcolr/gs;
	$panel =~ s/WFONTFACE/$wfontface/gs;
	$panel =~ s/WFONTSIZE/$wfontsize/gs;

	# Scrollback/Remote Control
	$panel =~ s/WSCRLBACK/$wscrlback/gs;
	$panel =~ s/WSCRLCTRL/$wscrlctrl/gs;


	# Framed, streaming, autoscrolling 
	if ($choice != 0)
	{
		# Frame sensitive information
		$param->{'COMM'} = 'True';
	}

	# HTML
	# Start form
	print $cgi->start_form();
	print $param->EmbedNormal($cgi, %{$param}), "\n"; 


	# Display HTML
	print "$panel\n";


	# End of form
	print $cgi->end_form(), "\n";

	# Print end of html if needed
	if ($choice != 0)
	{ 
		# Retreive proper footer
		print $database->FetchRealmFooter($param->{'ROOM'}); 
	}
}

# Alias for dependency
sub CommDependant
{ Comm(shift, shift, 0); }

# Alias for independnt framed
sub CommIndependant
{ Comm(shift, shift, 1); }

# Alias for independant streaming
sub CommStream
{ Comm(shift, shift, 2); }

# Alias for independant auto-scrolling streaming
sub CommAutoScroll
{ Comm(shift, shift, 3); }


#####################
# DataWrite
#
# Encharged with properly handling information and actually making posts
# once formatted inserting them into the database.

sub DataWrite							# Database, param
{
        #####################
        # Data members
	my $self     = shift;					# Self referencing link
	my $param    = shift;					# Parameter handle

	my $database = $self->{'HANDLE'};			# Database handle
	my $data     = $database->{'HANDLE'};			# CGI handle

	my $locate;						# Location handler

	my %realm;						# Realm hash


        #####################
        # Data members

	# Assign to target if undefined
	$param->{'TARGET'} = (defined($param->{'TARGET'})) ? $param->{'TARGET'} : 'all';

	# Only do following if posts exist
	if ((defined($param->{'POST'})) && ((length($param->{'POST'}) > 1)))
	{
		# Link DBM to hash
		$database->GetHashRealm(\%realm);


		# Update if not a dice roll or system wide post
		if (($param->{'TARGET'} ne 'dice roll') && ($param->{'TARGET'} ne $realm{'TxtSystem'}))
		{
			# Determine location
			# Location handling
			if (defined($param->{'LOCL'}))
			{
				# Valid entry
				if (($param->{'LOCL'} ne $realm{'TxtNone'})
				 && (length($param->{'LOCL'}) > 2))	
				{
					# Direct association
					($locate) = $param->{'LOCL'};	
				}

				# Invalid
				else
				{
					# Cleanup 
					$param->Cleanup('LOCL');
				}

			}

			# Re-assign defaults
			else
			{
				# Use defaults unless you can reset
				($locate) = $database->DataGetPuppetLocale($param->{'CHAR'});

				# Cleanup
				$param->Cleanup('LOCL');
			}


			# Update database entry
			$data->do("UPDATE Puppet 
				SET PuppetTextColour=?, 
				    PuppetTextSize=?,
				    PuppetTextFace=?, 
				    PuppetLocale=?
				WHERE PuppetName=?
				  AND PuppeteerLogin=?", {}, 

			 $param->{'COLOR'}, 
		 	 $param->{'SIZE'},
			 $param->{'FACE'}, 
			 $locate,
			 $param->{'CHAR'},
			 $param->{'USER'});
		}

		# Handle posts
		$self->PostHandler($param, \%realm); 
	}
	
	# No post simply post for timestamp purposes
	else
	{
		$data->do("INSERT INTO Post 
			(PostIDNumber,
			 RealmName, 
			 PuppetName,
			 PuppeteerLogin,
			 PostTo,
			 PostTimeStamp)
			VALUES(NULL,?,?,?,?,?)", {}, 

		 $param->{'ROOM'}, 
		 $param->{'CHAR'},
		 $param->{'USER'}, 
		 'system',
		 time);
	}
}


#####################
# FontSelect
#
# Based on the ability to allow hosts to select which colours, fonts faces and 
# sizes they wish to allow.  This sub-routine will determin what to send
# back.  As being either the hosts entity or a complete list as default.

sub FontSelect
{
        #####################
        # Data members
        my $self     = shift;                                   # Self referencing link
        my $param    = shift;                                   # Parameter handle
	my $type     = shift;					# Font type

        my $database = $self->{'HANDLE'};                       # Database handle

	my $options;						# String of options
	my @options;						# List of options


	#####################
	# Program Area

	# Pull realm values	
	($options) = $database->DataGet("SELECT RealmFont$type 
		FROM  Realm
		WHERE RealmName=?", $param->{'ROOM'});

	# Valid or not
	unless (defined($options))
	{
		# Retreive list
		$database->GetList(\@options, "SELECT FontName
			FROM Font
			WHERE FontType=?
			ORDER BY FontCode", $type);
	}
	else
	{
		# Seperate into useable text
		@options = split(/:/, $options);
	}

	# Return defaults
	return @options;
}

# Alias for font color
sub FontSelectColor
{ return FontSelect(shift, shift, 'Color'); }

# Alias for font face
sub FontSelectFace
{ return FontSelect(shift, shift, 'Face'); }

# Alias for font size
sub FontSelectSize
{ return FontSelect(shift, shift, 'Size'); }


####################
# Gummy Handler
#
# This will control how a gummy is shown.   This will vary depending on access
# and which real they access at the time.  Since such behavior is for 
# moderators or supervisors only this will not be triggered often.

sub GummyHandler
{
	#####################
	# Data members
	my $self     = shift;					# Self referencing link
	my $database = shift;					# Database handle
	my $param    = shift;					# Parameter handle
	my $realm    = shift;					# Realm handle
	my $gummy    = shift;					# Gummy handling

	my $admin;						# Administrator	
	my $mod;						# Moderator
	my $super;						# Supervisor

	my $class;						# Gummy class
	my $choice;						# Choice of the matter


	#####################
	# Program Area

	# Only if defined
	if (defined($gummy))
	{

		# Retreive privs
		my ($super, $admin, $mod) = $database->DataGetPrivs($param->{'USER'});

		# Only if supervisory or moderator
		if (($super eq 'yes') || ($mod eq 'yes'))
		{
			# Retreive access
			my ($choice) = $database->DataGet("SELECT PuppeteerLogin
				FROM  RealmSupervisor
				WHERE RealmName=?
				  AND PuppeteerLogin=?", 

			 $param->{'ROOM'},
			 $param->{'USER'});

			# If in a restricted realm
			unless (defined($choice))
			{
				# Retreive gummy class
				($class) = $database->DataGetGummyClass($gummy);

				# Is gummy supervisor or moderator
				if (($class eq 'super') 
				 or ($class eq 'mod'))
				{
					# Display only default
					$gummy = $realm->{SetGummyDefault};
				}
			}
		}
	}
		
	# Gummy assignment
	$gummy = (defined($gummy)) ? $gummy : $realm->{SetGummyDefault};  

	# Return
	return $gummy;
}


#####################
# Post Handler
#
# Depending on rules will handle the formatting of the three styles of posts
# filter language and HTML if needed or specified.

sub PostHandler
{
	#####################
	# Data members
	my $self     = shift;					# Self referencing link
	my $param    = shift;					# Parameter handle
	my $realm    = shift;					# Reference to realm

	my $cgi      = $self->{'CGI'};				# CGI handle
	my $database = $self->{'HANDLE'};			# Database handle
	my $data     = $database->{'HANDLE'};			# Database connection handle

	my $p_handle;						# Puppet information
	my $r_handle;						# Realm information

	my $date     = new Ethereal::Date();			# Date handler
	my $filter   = new Ethereal::Filter();			# Filter handler
	my $option   = new Ethereal::Option();			# Option handler

	my $p_res;						# Puppet results
	my $r_res;						# Realm results

	my $full;						# Full formating
	my $imageless;						# No images
	my $spartan;						# Spartan formatting

	my $img_avg;						# Average shrinking
	my $img_height;						# Image height
	my $img_width;						# Image width

	my $bio;						# Bio link
	my $gummy;						# Gummy link
	my $image;						# Image section
	my $inline;						# Inline parameters
	my $privs;						# Room privs
	my $tabletop;						# Table top 
	my $tag;						# Tagline
	my $title;						# Title

	my $post;						# Actual post
	my $position = 0;					# Position of post

	my $start;						# Start of table
	my $end;						# End of table

	my $time     = $date->GetDate(time);			# Date and Time
	my $target   = $param->{'TARGET'};			# Target to

	my @lang;						# Language array
	my @posts;						# Array of posts
	my @realmpic;						# Ream picture restriction


	#####################
	# Program area

	# Retreive puppet and realm information
	# Puppet information
	$p_handle = $data->prepare("SELECT 
		  PuppetName,
		  PuppeteerLogin, 
		  PuppetTag,
		  PuppetPic,
		  PuppetPicHeight,
		  PuppetPicWidth,
		  PuppetPicSize,
		  PuppetTitle, 
		  PuppetGummy 
		FROM  Puppet 
		WHERE PuppetName=?
		  AND PuppeteerLogin=?");

	# Realm Information
	$r_handle  = $data->prepare("SELECT 
		  RealmLanguage, 
		  RealmTags 
		FROM  Realm 
		WHERE RealmName=?");

	# Execute
	$p_handle->execute($param->{'CHAR'}, $param->{'USER'});
	$r_handle->execute($param->{'ROOM'});
	
	# Associate to hash
	$p_res = $p_handle->fetchrow_hashref();
	$r_res = $r_handle->fetchrow_hashref(); 	


	# Determine colour to use in frames
	if ($target eq 'all')
	{ $tabletop = "WIDTH=\"100%\""; }
	elsif ($target eq $realm->{'TxtSystem'})
	{ $tabletop = "BORDER=\"$realm->{SetBorderSize}\" WIDTH=\"100%\">\n\t<TR BGCOLOR=\"$realm->{SetBackSystem}\">\n\t\t<TD>$realm->{TagHeadSystem}</TD>\n\t<TR"; }
	elsif ($target eq 'dice roll')
	{ $tabletop = "BORDER=\"$realm->{SetBorderSize}\" WIDTH=\"100%\">\n\t<TR BGCOLOR=\"$realm->{SetBackDice}\">\n\t\t<TD>$realm->{TagHeadDice}</TD>\n\t<TR"; }
	elsif ($target ne 'system')
	{ $tabletop = "BORDER=\"$realm->{SetBorderSize}\" WIDTH=\"100%\">\n\t<TR BGCOLOR=\"$realm->{SetBackPrivate}\">\n\t\t<TD>$realm->{TagHeadPrivate}</TD>\n\t<TR"; }

	# Language filtering
	if ($r_res->{'RealmLanguage'} eq 'restricted')
	{
		# Retreive words
		$database->GetListWords(\@lang, $param->{'ROOM'});

		# Run through
		my $word;

		while ($word = shift(@lang))
		{
			# Run a global replace
			$param->{'POST'} =~ s/$word/****/gsi;
		}
	}

	# Bio link
	# Determin gummy
	$gummy = $self->GummyHandler($database, $param, $realm, $p_res->{'PuppetGummy'});


	# Inline and format
	$inline = $param->EmbedInline($cgi, 'CHAR', $param->{'CHAR'}, 'USER', $param->{'USER'});
	$bio = "<A HREF=\"$realm->{'LnkBio'}$inline\" TARGET=\"_blank\"><IMG SRC=\"$gummy\" BORDER=\"0\" ALIGN=\"LEFT\"></A>";

	# Format title
	if (defined($p_res->{'PuppetTitle'}))
	{ 
		# Verify if NAME attribute exists
		if ($p_res->{'PuppetTitle'} =~ /NAME/)
		{
			# Prevent tag embed
			$p_res->{'PuppetTitle'} =~ s/<NAME>/NAME/g;

			# Replace with puppet name
			$p_res->{'PuppetTitle'} =~ s/NAME/$param->{'CHAR'}/g;
			$title = $p_res->{'PuppetTitle'};
		}

		# Otherwise prepend name
		else
		{ $title = "$param->{CHAR} $p_res->{PuppetTitle}"; }
	}
	

	# Assign chacter name to title
	else
	{ $title = $param->{'CHAR'}; }	

	# Create formatted line
	$title = "$bio <B><FONT SIZE=\"$realm->{'SetTitleSize'}\">$title</FONT></B>";


	# Tagline handling
	$tag = (defined($p_res->{'PuppetTag'})) ? "(<I>$puppet$p_res->{'PuppetTag'}</I>)" : "";

	# Create tagline
	unless (($target ne 'all') 
            &&  ($target ne 'system')
	    &&  ($target ne 'dice roll')
	    &&  ($target ne $realm->{'TxtSystem'}))
	{ $tag   = "$tag ($time)"; }
	else
	{ $tag   = "$tag ($realm->{'TagPuppetPMFrom'}: $param->{'CHAR'}) ($realm->{'TagPuppetPMTo'}: $param->{'OTARGET'}) ($time)"; }
		

	# Determine font colour, face and size
	my ($fontc) = $database->DataGetFontCode($param->{'COLOR'}, 'color');
	my ($fontf) = $database->DataGetFontCode($param->{'FACE'}, 'face');
	my ($fonts) = $database->DataGetFontCode($param->{'SIZE'}, 'size');

	# Image handling
	$image = (defined($p_res->{'PuppetPic'})) ?  "<A HREF=\"$p_res->{'PuppetPic'}\" TARGET=\"_TOP\">$realm->{'TagLnkImage'}</A><BR>" : ""; 


	# Filtering of the text
	# Filter HTML if option is selected
	if ($r_res->{'RealmTags'} eq 'restricted')
	{ $param->{'POST'} =~ s/<[^>]*>//gs; }

	# Standard filtering
	else
	{ ($param->{'POST'}) = $filter->SafeHTML($param->{'POST'}); }


	# Pre-post location
	if (defined($param->{'LOCL'}))
	{
		# Add additional location tag
		$param->{'POST'} = "<B>~~$param->{LOCL}~~</B>\n\n$param->{POST}";
	}

	# Large post handling
	# Divide and conquer
	if ((length($param->{'POST'})) > 10240)
	{
		# Cannot allow stray HTML tags
		$param->{'POST'} =~ s/<[^>]*>//gs;

		# Prepend beginning of posts
		$post = substr($param->{'POST'}, 0, 10240);
		push(@posts, "$post ...");

		$position++;

		# Truncate into edible bites
		while (length(substr($param->{'POST'}, ($position * 10240), 10241)) > 10240)
		{
			$post = substr($param->{'POST'}, ($position * 10240), 10240);
			push(@posts, "... $post ...");

			$position++;
		}

		# Append whatever is left
		$post = substr($param->{'POST'}, ($position * 10240), 10240);
		push(@posts, "... $post");
	}
	# Pass as first element of array
	else
	{ push(@posts, "$param->{'POST'}"); }


	# Clear post
	$param->Cleanup('POST');


	# Cycle though individual sections
	foreach $post (@posts)
	{

		# Handling of links
		$post      =~ s/$/\n/;
		$post      =~ s/(http:\/\/.*?)[\s]/<A HREF="$1" TARGET="_blank">$1<\/A> /gm;

		# Add colors
		$post = "<FONT SIZE=\"$fonts\" COLOR=\"$fontc\" FACE=\"$fontf\">$post</FONT>";

		# Full formatting
		if (defined($p_res->{'PuppetPic'}))
		{
			# Retreive image size information
			my ($p_height, $p_width, $p_size) = $database->DataGetRealmImageInfo($param->{'ROOM'});

			# Format image format accordingly
			if (($p_size != 0) && ($p_res->{'PuppetPicSize'} > $p_size)) 
			{
				# Imageless format 
				$full = "$image$title $tag<BR><BR>$post"; 
			}
			else
			{
				# Define images
				$img_height = $p_res->{'PuppetPicHeight'};
				$img_width  = $p_res->{'PuppetPicWidth'};

				# Ignore non restricted realms
				if (($p_height != 0) && ($p_width != 0))
				{
					# Height handling
					if ($img_height > $p_height)
					{
						# Get average
						$img_avg = $p_height / $img_height;

						# Forceful resize
						$img_height = int($img_height * $img_avg);
						$img_width  = int($img_width  * $img_avg);
					}

					# Width handling
					if ($img_width > $p_width)
					{
						# Get average
						$img_avg = $p_width / $img_width;

						# Forceful resize
						$img_height = int($img_height * $img_avg);
						$img_width  = int($img_width  * $img_avg);
					}
				}

				# Generate link
 				$full = "<IMG SRC=\"$p_res->{PuppetPic}\" HEIGHT=\"$img_height\" WIDTH=\"$img_width\" ALIGN=\"left\" BORDER=\"0\">$title $tag<BR><BR>$post"; 
			}
		}
		else
		{ $full = "$title $tag<BR><BR>$post"; }
	

		# Format non imaged format
		$imageless = "$image$title $tag<BR><BR>$post";


		# Create spartan format
		$post =~ s/<[^>]*>//gs;
		$spartan = "$image$title $tag<BR><BR>$post<BR><BR>";


		# Embed all formats in tables
		$start = "<BR>\n<TABLE $tabletop>\n\t<TR>\n\t\t<TD>";
		$end   = "</TD>\n\t</TR>\n</TABLE>\n";

		# Convert \n to a useable character	
		$full      =~ s/\n/<BR>/gs;
		$imageless =~ s/\n/<BR>/gs;
		$spartan   =~ s/\n/<BR>/gs;

		# Assembly
		$full      = "$start$full$end";
		$imageless = "$start$imageless$end";
		$spartan   = "$start$spartan$end";


		# Normal messages
		if (($target eq 'all') || ($target eq 'dice roll'))
		{
			# Insert into database
			$data->do("INSERT INTO Post 
				(PostIDNumber,
				 RealmName,
				 PuppetName,
				 PuppeteerLogin,
				 PostTo,
				 PostTimestamp,
				 PostFullFormat,
				 PostImagelessFormat,
				 PostPartialFormat)		 
				VALUES(NULL,?,?,?,?,?,?,?,?)", {}, 
	
			 $param->{'ROOM'}, 
			 $param->{'CHAR'}, 
			 $param->{'USER'}, 
			 'all', 
			 time, 
			 $full, 
			 $imageless, 
			 $spartan);
		}

		# System wide
		elsif ($target eq $realm->{'TxtSystem'})
		{
			#####################
			# Data Members

			my @realms;					# List of realms
			my $realm;					# Specific realm

			# Retreive lists of realms
			$database->GetListGlobalRealms(\@realms);

			# Run through each realm
			foreach $realm (@realms)
			{
				# Insert into database
				$data->do("INSERT INTO Post 
					(PostIDNumber,
					 RealmName,
					 PuppetName,
					 PuppeteerLogin,
					 PostTo,
					 PostTimestamp,
					 PostFullFormat,
					 PostImagelessFormat,
					 PostPartialFormat)
					VALUES(NULL,?,?,?,?,?,?,?,?)", {}, 
				 $realm, 
				 $param->{'CHAR'}, 
				 $param->{'USER'}, 
				 'all', 
				 time, 
				 $full, 
				 $imageless, 
				 $spartan);
			}
		}

		# Insert message to be returned to poster
		elsif ($param->{'TARGET'} ne 'system')
		{
			# Send to target
			$data->do("INSERT INTO Post 
				(PostIDNumber,
				 RealmName,
				 PuppetName,
				 PuppeteerLogin,
				 PostTo,
				 PostTimestamp,
				 PostFullFormat,
				 PostImagelessFormat,
				 PostPartialFormat)
				VALUES(NULL,?,?,?,?,?,?,?,?)", {}, 
			 $param->{'ROOM'}, 
			 $param->{'CHAR'}, 
			 $param->{'USER'}, 
			 $param->{'TARGET'}, 
			 time, 
			 $full, 
			 $imageless, 
			 $spartan);

			# Send back owner
			$data->do("INSERT INTO Post 
				(PostIDNumber,
				 RealmName,
				 PuppetName,
				 PuppeteerLogin,
				 PostTo,
				 PostTimestamp,
				 PostFullFormat,
				 PostImagelessFormat,
				 PostPartialFormat)
				VALUES(NULL,?,?,?,?,?,?,?,?)", {}, 
			 $param->{'ROOM'}, 
			 $param->{'CHAR'}, 
			 $param->{'USER'}, 
			 $param->{'SOURCE'}, 
			 time, 
			 $full, 
			 $imageless, 
			 $spartan);
		}
	}


	# End of handles
	$r_handle->finish();
	$p_handle->finish();
}

1;								# Return true
